@@ -1,14 +1,14 @@
 /* Do various things to symbol tables (other than lookup), for GDB.
 
-   Copyright 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
-   1995, 1996, 1997, 1998, 1999, 2000, 2002, 2003, 2004 Free Software
-   Foundation, Inc.
+   Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995,
+   1996, 1997, 1998, 1999, 2000, 2002, 2003, 2004, 2007, 2008, 2009, 2010
+   Free Software Foundation, Inc.
 
    This file is part of GDB.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -17,9 +17,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #include "defs.h"
 #include "symtab.h"
@@ -30,15 +28,19 @@
 #include "breakpoint.h"
 #include "command.h"
 #include "gdb_obstack.h"
+#include "exceptions.h"
 #include "language.h"
 #include "bcache.h"
 #include "block.h"
 #include "gdb_regex.h"
+#include "gdb_stat.h"
 #include "dictionary.h"
 
 #include "gdb_string.h"
 #include "readline/readline.h"
 
+#include "psymtab.h"
+
 #ifndef DEV_TTY
 #define DEV_TTY "/dev/tty"
 #endif
@@ -58,69 +60,34 @@ FILE *std_err;
 static void dump_symtab (struct objfile *, struct symtab *,
 			 struct ui_file *);
 
-static void dump_psymtab (struct objfile *, struct partial_symtab *,
-			  struct ui_file *);
-
 static void dump_msymbols (struct objfile *, struct ui_file *);
 
 static void dump_objfile (struct objfile *);
 
 static int block_depth (struct block *);
 
-static void print_partial_symbols (struct partial_symbol **, int,
-				   char *, struct ui_file *);
-
-static void free_symtab_block (struct objfile *, struct block *);
-
 void _initialize_symmisc (void);
 
 struct print_symbol_args
   {
+    struct gdbarch *gdbarch;
     struct symbol *symbol;
     int depth;
     struct ui_file *outfile;
   };
 
 static int print_symbol (void *);
-
-static void free_symtab_block (struct objfile *, struct block *);
 
-
-/* Free a struct block <- B and all the symbols defined in that block.  */
-
-/* FIXME: carlton/2003-04-28: I don't believe this is currently ever
-   used.  */
-
-static void
-free_symtab_block (struct objfile *objfile, struct block *b)
-{
-  struct dict_iterator iter;
-  struct symbol *sym;
-
-  ALL_BLOCK_SYMBOLS (b, iter, sym)
-    {
-      xfree (DEPRECATED_SYMBOL_NAME (sym));
-      xfree (sym);
-    }
-
-  dict_free (BLOCK_DICT (b));
-  xfree (b);
-}
-
 /* Free all the storage associated with the struct symtab <- S.
-   Note that some symtabs have contents malloc'ed structure by structure,
-   while some have contents that all live inside one big block of memory,
-   and some share the contents of another symbol table and so you should
-   not free the contents on their behalf (except sometimes the linetable,
-   which maybe per symtab even when the rest is not).
+   Note that some symtabs have contents that all live inside one big block of
+   memory, and some share the contents of another symbol table and so you
+   should not free the contents on their behalf (except sometimes the
+   linetable, which maybe per symtab even when the rest is not).
    It is s->free_code that says which alternative to use.  */
 
 void
 free_symtab (struct symtab *s)
 {
-  int i, n;
-  struct blockvector *bv;
-
   switch (s->free_code)
     {
     case free_nothing:
@@ -129,18 +96,6 @@ free_symtab (struct symtab *s)
          Therefore, do nothing.  */
       break;
 
-    case free_contents:
-      /* Here all the contents were malloc'ed structure by structure
-         and must be freed that way.  */
-      /* First free the blocks (and their symbols.  */
-      bv = BLOCKVECTOR (s);
-      n = BLOCKVECTOR_NBLOCKS (bv);
-      for (i = 0; i < n; i++)
-	free_symtab_block (s->objfile, BLOCKVECTOR_BLOCK (bv, i));
-      /* Free the blockvector itself.  */
-      xfree (bv);
-      /* Also free the linetable.  */
-
     case free_linetable:
       /* Everything will be freed either by our `free_func'
          or by some other symtab, except for our linetable.
@@ -167,13 +122,17 @@ free_symtab (struct symtab *s)
 void
 print_symbol_bcache_statistics (void)
 {
+  struct program_space *pspace;
   struct objfile *objfile;
 
   immediate_quit++;
-  ALL_OBJFILES (objfile)
+  ALL_PSPACES (pspace)
+    ALL_PSPACE_OBJFILES (pspace, objfile)
   {
-    printf_filtered ("Byte cache statistics for '%s':\n", objfile->name);
+    printf_filtered (_("Byte cache statistics for '%s':\n"), objfile->name);
     print_bcache_statistics (objfile->psymbol_cache, "partial symbol cache");
+    print_bcache_statistics (objfile->macro_cache, "preprocessor macro cache");
+    print_bcache_statistics (objfile->filename_cache, "file name cache");
   }
   immediate_quit--;
 }
@@ -181,37 +140,33 @@ print_symbol_bcache_statistics (void)
 void
 print_objfile_statistics (void)
 {
+  struct program_space *pspace;
   struct objfile *objfile;
   struct symtab *s;
-  struct partial_symtab *ps;
   int i, linetables, blockvectors;
 
   immediate_quit++;
-  ALL_OBJFILES (objfile)
+  ALL_PSPACES (pspace)
+    ALL_PSPACE_OBJFILES (pspace, objfile)
   {
-    printf_filtered ("Statistics for '%s':\n", objfile->name);
+    printf_filtered (_("Statistics for '%s':\n"), objfile->name);
     if (OBJSTAT (objfile, n_stabs) > 0)
-      printf_filtered ("  Number of \"stab\" symbols read: %d\n",
+      printf_filtered (_("  Number of \"stab\" symbols read: %d\n"),
 		       OBJSTAT (objfile, n_stabs));
     if (OBJSTAT (objfile, n_minsyms) > 0)
-      printf_filtered ("  Number of \"minimal\" symbols read: %d\n",
+      printf_filtered (_("  Number of \"minimal\" symbols read: %d\n"),
 		       OBJSTAT (objfile, n_minsyms));
     if (OBJSTAT (objfile, n_psyms) > 0)
-      printf_filtered ("  Number of \"partial\" symbols read: %d\n",
+      printf_filtered (_("  Number of \"partial\" symbols read: %d\n"),
 		       OBJSTAT (objfile, n_psyms));
     if (OBJSTAT (objfile, n_syms) > 0)
-      printf_filtered ("  Number of \"full\" symbols read: %d\n",
+      printf_filtered (_("  Number of \"full\" symbols read: %d\n"),
 		       OBJSTAT (objfile, n_syms));
     if (OBJSTAT (objfile, n_types) > 0)
-      printf_filtered ("  Number of \"types\" defined: %d\n",
+      printf_filtered (_("  Number of \"types\" defined: %d\n"),
 		       OBJSTAT (objfile, n_types));
-    i = 0;
-    ALL_OBJFILE_PSYMTABS (objfile, ps)
-      {
-        if (ps->readin == 0)
-          i++;
-      }
-    printf_filtered ("  Number of psym tables (not yet expanded): %d\n", i);
+    if (objfile->sf)
+      objfile->sf->qf->print_stats (objfile);
     i = linetables = blockvectors = 0;
     ALL_OBJFILE_SYMTABS (objfile, s)
       {
@@ -221,21 +176,23 @@ print_objfile_statistics (void)
         if (s->primary == 1)
           blockvectors++;
       }
-    printf_filtered ("  Number of symbol tables: %d\n", i);
-    printf_filtered ("  Number of symbol tables with line tables: %d\n", 
+    printf_filtered (_("  Number of symbol tables: %d\n"), i);
+    printf_filtered (_("  Number of symbol tables with line tables: %d\n"), 
                      linetables);
-    printf_filtered ("  Number of symbol tables with blockvectors: %d\n", 
+    printf_filtered (_("  Number of symbol tables with blockvectors: %d\n"), 
                      blockvectors);
     
     if (OBJSTAT (objfile, sz_strtab) > 0)
-      printf_filtered ("  Space used by a.out string tables: %d\n",
+      printf_filtered (_("  Space used by a.out string tables: %d\n"),
 		       OBJSTAT (objfile, sz_strtab));
-    printf_filtered ("  Total memory used for objfile obstack: %d\n",
+    printf_filtered (_("  Total memory used for objfile obstack: %d\n"),
 		     obstack_memory_used (&objfile->objfile_obstack));
-    printf_filtered ("  Total memory used for psymbol cache: %d\n",
+    printf_filtered (_("  Total memory used for psymbol cache: %d\n"),
 		     bcache_memory_used (objfile->psymbol_cache));
-    printf_filtered ("  Total memory used for macro cache: %d\n",
+    printf_filtered (_("  Total memory used for macro cache: %d\n"),
 		     bcache_memory_used (objfile->macro_cache));
+    printf_filtered (_("  Total memory used for file name cache: %d\n"),
+		     bcache_memory_used (objfile->filename_cache));
   }
   immediate_quit--;
 }
@@ -244,7 +201,6 @@ static void
 dump_objfile (struct objfile *objfile)
 {
   struct symtab *symtab;
-  struct partial_symtab *psymtab;
 
   printf_filtered ("\nObject file %s:  ", objfile->name);
   printf_filtered ("Objfile at ");
@@ -254,25 +210,8 @@ dump_objfile (struct objfile *objfile)
   printf_filtered (", %d minsyms\n\n",
 		   objfile->minimal_symbol_count);
 
-  if (objfile->psymtabs)
-    {
-      printf_filtered ("Psymtabs:\n");
-      for (psymtab = objfile->psymtabs;
-	   psymtab != NULL;
-	   psymtab = psymtab->next)
-	{
-	  printf_filtered ("%s at ",
-			   psymtab->filename);
-	  gdb_print_host_address (psymtab, gdb_stdout);
-	  printf_filtered (", ");
-	  if (psymtab->objfile != objfile)
-	    {
-	      printf_filtered ("NOT ON CHAIN!  ");
-	    }
-	  wrap_here ("  ");
-	}
-      printf_filtered ("\n\n");
-    }
+  if (objfile->sf)
+    objfile->sf->qf->dump (objfile);
 
   if (objfile->symtabs)
     {
@@ -299,6 +238,7 @@ dump_objfile (struct objfile *objfile)
 static void
 dump_msymbols (struct objfile *objfile, struct ui_file *outfile)
 {
+  struct gdbarch *gdbarch = get_objfile_arch (objfile);
   struct minimal_symbol *msymbol;
   int index;
   char ms_type;
@@ -309,10 +249,12 @@ dump_msymbols (struct objfile *objfile, struct ui_file *outfile)
       fprintf_filtered (outfile, "No minimal symbols found.\n");
       return;
     }
-  for (index = 0, msymbol = objfile->msymbols;
-       DEPRECATED_SYMBOL_NAME (msymbol) != NULL; msymbol++, index++)
+  index = 0;
+  ALL_OBJFILE_MSYMBOLS (objfile, msymbol)
     {
-      switch (msymbol->type)
+      struct obj_section *section = SYMBOL_OBJ_SECTION (msymbol);
+
+      switch (MSYMBOL_TYPE (msymbol))
 	{
 	case mst_unknown:
 	  ms_type = 'u';
@@ -346,104 +288,38 @@ dump_msymbols (struct objfile *objfile, struct ui_file *outfile)
 	  break;
 	}
       fprintf_filtered (outfile, "[%2d] %c ", index, ms_type);
-      print_address_numeric (SYMBOL_VALUE_ADDRESS (msymbol), 1, outfile);
-      fprintf_filtered (outfile, " %s", DEPRECATED_SYMBOL_NAME (msymbol));
-      if (SYMBOL_BFD_SECTION (msymbol))
+      fputs_filtered (paddress (gdbarch, SYMBOL_VALUE_ADDRESS (msymbol)),
+		      outfile);
+      fprintf_filtered (outfile, " %s", SYMBOL_LINKAGE_NAME (msymbol));
+      if (section)
 	fprintf_filtered (outfile, " section %s",
 			  bfd_section_name (objfile->obfd,
-					    SYMBOL_BFD_SECTION (msymbol)));
+					    section->the_bfd_section));
       if (SYMBOL_DEMANGLED_NAME (msymbol) != NULL)
 	{
 	  fprintf_filtered (outfile, "  %s", SYMBOL_DEMANGLED_NAME (msymbol));
 	}
-#ifdef SOFUN_ADDRESS_MAYBE_MISSING
       if (msymbol->filename)
 	fprintf_filtered (outfile, "  %s", msymbol->filename);
-#endif
       fputs_filtered ("\n", outfile);
+      index++;
     }
   if (objfile->minimal_symbol_count != index)
     {
-      warning ("internal error:  minimal symbol count %d != %d",
+      warning (_("internal error:  minimal symbol count %d != %d"),
 	       objfile->minimal_symbol_count, index);
     }
   fprintf_filtered (outfile, "\n");
 }
 
 static void
-dump_psymtab (struct objfile *objfile, struct partial_symtab *psymtab,
-	      struct ui_file *outfile)
-{
-  int i;
-
-  fprintf_filtered (outfile, "\nPartial symtab for source file %s ",
-		    psymtab->filename);
-  fprintf_filtered (outfile, "(object ");
-  gdb_print_host_address (psymtab, outfile);
-  fprintf_filtered (outfile, ")\n\n");
-  fprintf_unfiltered (outfile, "  Read from object file %s (",
-		      objfile->name);
-  gdb_print_host_address (objfile, outfile);
-  fprintf_unfiltered (outfile, ")\n");
-
-  if (psymtab->readin)
-    {
-      fprintf_filtered (outfile,
-			"  Full symtab was read (at ");
-      gdb_print_host_address (psymtab->symtab, outfile);
-      fprintf_filtered (outfile, " by function at ");
-      gdb_print_host_address (psymtab->read_symtab, outfile);
-      fprintf_filtered (outfile, ")\n");
-    }
-
-  fprintf_filtered (outfile, "  Relocate symbols by ");
-  for (i = 0; i < psymtab->objfile->num_sections; ++i)
-    {
-      if (i != 0)
-	fprintf_filtered (outfile, ", ");
-      wrap_here ("    ");
-      print_address_numeric (ANOFFSET (psymtab->section_offsets, i),
-			     1,
-			     outfile);
-    }
-  fprintf_filtered (outfile, "\n");
-
-  fprintf_filtered (outfile, "  Symbols cover text addresses ");
-  print_address_numeric (psymtab->textlow, 1, outfile);
-  fprintf_filtered (outfile, "-");
-  print_address_numeric (psymtab->texthigh, 1, outfile);
-  fprintf_filtered (outfile, "\n");
-  fprintf_filtered (outfile, "  Depends on %d other partial symtabs.\n",
-		    psymtab->number_of_dependencies);
-  for (i = 0; i < psymtab->number_of_dependencies; i++)
-    {
-      fprintf_filtered (outfile, "    %d ", i);
-      gdb_print_host_address (psymtab->dependencies[i], outfile);
-      fprintf_filtered (outfile, " %s\n",
-			psymtab->dependencies[i]->filename);
-    }
-  if (psymtab->n_global_syms > 0)
-    {
-      print_partial_symbols (objfile->global_psymbols.list
-			     + psymtab->globals_offset,
-			     psymtab->n_global_syms, "Global", outfile);
-    }
-  if (psymtab->n_static_syms > 0)
-    {
-      print_partial_symbols (objfile->static_psymbols.list
-			     + psymtab->statics_offset,
-			     psymtab->n_static_syms, "Static", outfile);
-    }
-  fprintf_filtered (outfile, "\n");
-}
-
-static void
-dump_symtab (struct objfile *objfile, struct symtab *symtab,
-	     struct ui_file *outfile)
+dump_symtab_1 (struct objfile *objfile, struct symtab *symtab,
+	       struct ui_file *outfile)
 {
+  struct gdbarch *gdbarch = get_objfile_arch (objfile);
   int i;
   struct dict_iterator iter;
-  int len, blen;
+  int len;
   struct linetable *l;
   struct blockvector *bv;
   struct symbol *sym;
@@ -468,7 +344,7 @@ dump_symtab (struct objfile *objfile, struct symtab *symtab,
       for (i = 0; i < len; i++)
 	{
 	  fprintf_filtered (outfile, " line %d at ", l->item[i].line);
-	  print_address_numeric (l->item[i].pc, 1, outfile);
+	  fputs_filtered (paddress (gdbarch, l->item[i].pc), outfile);
 	  fprintf_filtered (outfile, "\n");
 	}
     }
@@ -496,31 +372,32 @@ dump_symtab (struct objfile *objfile, struct symtab *symtab,
 	     wants it.  */
 	  fprintf_filtered (outfile, ", %d syms/buckets in ",
 			    dict_size (BLOCK_DICT (b)));
-	  print_address_numeric (BLOCK_START (b), 1, outfile);
+	  fputs_filtered (paddress (gdbarch, BLOCK_START (b)), outfile);
 	  fprintf_filtered (outfile, "..");
-	  print_address_numeric (BLOCK_END (b), 1, outfile);
+	  fputs_filtered (paddress (gdbarch, BLOCK_END (b)), outfile);
 	  if (BLOCK_FUNCTION (b))
 	    {
-	      fprintf_filtered (outfile, ", function %s", DEPRECATED_SYMBOL_NAME (BLOCK_FUNCTION (b)));
+	      fprintf_filtered (outfile, ", function %s",
+				SYMBOL_LINKAGE_NAME (BLOCK_FUNCTION (b)));
 	      if (SYMBOL_DEMANGLED_NAME (BLOCK_FUNCTION (b)) != NULL)
 		{
 		  fprintf_filtered (outfile, ", %s",
 				SYMBOL_DEMANGLED_NAME (BLOCK_FUNCTION (b)));
 		}
 	    }
-	  if (BLOCK_GCC_COMPILED (b))
-	    fprintf_filtered (outfile, ", compiled with gcc%d", BLOCK_GCC_COMPILED (b));
 	  fprintf_filtered (outfile, "\n");
 	  /* Now print each symbol in this block (in no particular order, if
 	     we're using a hashtable).  */
 	  ALL_BLOCK_SYMBOLS (b, iter, sym)
 	    {
 	      struct print_symbol_args s;
+
+	      s.gdbarch = gdbarch;
 	      s.symbol = sym;
 	      s.depth = depth + 1;
 	      s.outfile = outfile;
 	      catch_errors (print_symbol, &s, "Error printing symbol:\n",
-			    RETURN_MASK_ALL);
+			    RETURN_MASK_ERROR);
 	    }
 	}
       fprintf_filtered (outfile, "\n");
@@ -531,6 +408,29 @@ dump_symtab (struct objfile *objfile, struct symtab *symtab,
     }
 }
 
+static void
+dump_symtab (struct objfile *objfile, struct symtab *symtab,
+	     struct ui_file *outfile)
+{
+  /* Set the current language to the language of the symtab we're dumping
+     because certain routines used during dump_symtab() use the current
+     language to print an image of the symbol.  We'll restore it later.
+     But use only real languages, not placeholders.  */
+  if (symtab->language != language_unknown
+      && symtab->language != language_auto)
+    {
+      enum language saved_lang;
+
+      saved_lang = set_language (symtab->language);
+
+      dump_symtab_1 (objfile, symtab, outfile);
+
+      set_language (saved_lang);
+    }
+  else
+    dump_symtab_1 (objfile, symtab, outfile);
+}
+
 void
 maintenance_print_symbols (char *args, int from_tty)
 {
@@ -546,13 +446,10 @@ maintenance_print_symbols (char *args, int from_tty)
 
   if (args == NULL)
     {
-      error ("\
-Arguments missing: an output file name and an optional symbol file name");
-    }
-  else if ((argv = buildargv (args)) == NULL)
-    {
-      nomem (0);
+      error (_("\
+Arguments missing: an output file name and an optional symbol file name"));
     }
+  argv = gdb_buildargv (args);
   cleanups = make_cleanup_freeargv (argv);
 
   if (argv[0] != NULL)
@@ -589,19 +486,22 @@ Arguments missing: an output file name and an optional symbol file name");
 static int
 print_symbol (void *args)
 {
+  struct gdbarch *gdbarch = ((struct print_symbol_args *) args)->gdbarch;
   struct symbol *symbol = ((struct print_symbol_args *) args)->symbol;
   int depth = ((struct print_symbol_args *) args)->depth;
   struct ui_file *outfile = ((struct print_symbol_args *) args)->outfile;
+  struct obj_section *section = SYMBOL_OBJ_SECTION (symbol);
 
   print_spaces (depth, outfile);
   if (SYMBOL_DOMAIN (symbol) == LABEL_DOMAIN)
     {
       fprintf_filtered (outfile, "label %s at ", SYMBOL_PRINT_NAME (symbol));
-      print_address_numeric (SYMBOL_VALUE_ADDRESS (symbol), 1, outfile);
-      if (SYMBOL_BFD_SECTION (symbol))
+      fputs_filtered (paddress (gdbarch, SYMBOL_VALUE_ADDRESS (symbol)),
+		      outfile);
+      if (section)
 	fprintf_filtered (outfile, " section %s\n",
-		       bfd_section_name (SYMBOL_BFD_SECTION (symbol)->owner,
-					 SYMBOL_BFD_SECTION (symbol)));
+			  bfd_section_name (section->the_bfd_section->owner,
+					    section->the_bfd_section));
       else
 	fprintf_filtered (outfile, "\n");
       return 1;
@@ -619,7 +519,7 @@ print_symbol (void *args)
 			  ? "enum"
 		     : (TYPE_CODE (SYMBOL_TYPE (symbol)) == TYPE_CODE_STRUCT
 			? "struct" : "union")),
-			    DEPRECATED_SYMBOL_NAME (symbol));
+			    SYMBOL_LINKAGE_NAME (symbol));
 	  LA_PRINT_TYPE (SYMBOL_TYPE (symbol), "", outfile, 1, depth);
 	}
       fprintf_filtered (outfile, ";\n");
@@ -652,6 +552,7 @@ print_symbol (void *args)
 	  {
 	    unsigned i;
 	    struct type *type = check_typedef (SYMBOL_TYPE (symbol));
+
 	    fprintf_filtered (outfile, "const %u hex bytes:",
 			      TYPE_LENGTH (type));
 	    for (i = 0; i < TYPE_LENGTH (type); i++)
@@ -662,22 +563,20 @@ print_symbol (void *args)
 
 	case LOC_STATIC:
 	  fprintf_filtered (outfile, "static at ");
-	  print_address_numeric (SYMBOL_VALUE_ADDRESS (symbol), 1, outfile);
-	  if (SYMBOL_BFD_SECTION (symbol))
+	  fputs_filtered (paddress (gdbarch, SYMBOL_VALUE_ADDRESS (symbol)),
+			  outfile);
+	  if (section)
 	    fprintf_filtered (outfile, " section %s",
-			      bfd_section_name
-			      (SYMBOL_BFD_SECTION (symbol)->owner,
-			       SYMBOL_BFD_SECTION (symbol)));
-	  break;
-
-	case LOC_INDIRECT:
-	  fprintf_filtered (outfile, "extern global at *(");
-	  print_address_numeric (SYMBOL_VALUE_ADDRESS (symbol), 1, outfile);
-	  fprintf_filtered (outfile, "),");
+			      bfd_section_name (section->the_bfd_section->owner,
+						section->the_bfd_section));
 	  break;
 
 	case LOC_REGISTER:
-	  fprintf_filtered (outfile, "register %ld", SYMBOL_VALUE (symbol));
+	  if (SYMBOL_IS_ARGUMENT (symbol))
+	    fprintf_filtered (outfile, "parameter register %ld",
+			      SYMBOL_VALUE (symbol));
+	  else
+	    fprintf_filtered (outfile, "register %ld", SYMBOL_VALUE (symbol));
 	  break;
 
 	case LOC_ARG:
@@ -685,19 +584,10 @@ print_symbol (void *args)
 			    SYMBOL_VALUE (symbol));
 	  break;
 
-	case LOC_LOCAL_ARG:
-	  fprintf_filtered (outfile, "arg at offset 0x%lx from fp",
-			    SYMBOL_VALUE (symbol));
-	  break;
-
 	case LOC_REF_ARG:
 	  fprintf_filtered (outfile, "reference arg at 0x%lx", SYMBOL_VALUE (symbol));
 	  break;
 
-	case LOC_REGPARM:
-	  fprintf_filtered (outfile, "parameter register %ld", SYMBOL_VALUE (symbol));
-	  break;
-
 	case LOC_REGPARM_ADDR:
 	  fprintf_filtered (outfile, "address parameter register %ld", SYMBOL_VALUE (symbol));
 	  break;
@@ -707,49 +597,37 @@ print_symbol (void *args)
 			    SYMBOL_VALUE (symbol));
 	  break;
 
-	case LOC_BASEREG:
-	  fprintf_filtered (outfile, "local at 0x%lx from register %d",
-			    SYMBOL_VALUE (symbol), SYMBOL_BASEREG (symbol));
-	  break;
-
-	case LOC_BASEREG_ARG:
-	  fprintf_filtered (outfile, "arg at 0x%lx from register %d",
-			    SYMBOL_VALUE (symbol), SYMBOL_BASEREG (symbol));
-	  break;
-
 	case LOC_TYPEDEF:
 	  break;
 
 	case LOC_LABEL:
 	  fprintf_filtered (outfile, "label at ");
-	  print_address_numeric (SYMBOL_VALUE_ADDRESS (symbol), 1, outfile);
-	  if (SYMBOL_BFD_SECTION (symbol))
+	  fputs_filtered (paddress (gdbarch, SYMBOL_VALUE_ADDRESS (symbol)),
+			  outfile);
+	  if (section)
 	    fprintf_filtered (outfile, " section %s",
-			      bfd_section_name
-			      (SYMBOL_BFD_SECTION (symbol)->owner,
-			       SYMBOL_BFD_SECTION (symbol)));
+			      bfd_section_name (section->the_bfd_section->owner,
+						section->the_bfd_section));
 	  break;
 
 	case LOC_BLOCK:
 	  fprintf_filtered (outfile, "block object ");
 	  gdb_print_host_address (SYMBOL_BLOCK_VALUE (symbol), outfile);
 	  fprintf_filtered (outfile, ", ");
-	  print_address_numeric (BLOCK_START (SYMBOL_BLOCK_VALUE (symbol)),
-				 1,
-				 outfile);
+	  fputs_filtered (paddress (gdbarch,
+				    BLOCK_START (SYMBOL_BLOCK_VALUE (symbol))),
+			  outfile);
 	  fprintf_filtered (outfile, "..");
-	  print_address_numeric (BLOCK_END (SYMBOL_BLOCK_VALUE (symbol)),
-				 1,
-				 outfile);
-	  if (SYMBOL_BFD_SECTION (symbol))
+	  fputs_filtered (paddress (gdbarch,
+				    BLOCK_END (SYMBOL_BLOCK_VALUE (symbol))),
+			  outfile);
+	  if (section)
 	    fprintf_filtered (outfile, " section %s",
-			      bfd_section_name
-			      (SYMBOL_BFD_SECTION (symbol)->owner,
-			       SYMBOL_BFD_SECTION (symbol)));
+			      bfd_section_name (section->the_bfd_section->owner,
+						section->the_bfd_section));
 	  break;
 
 	case LOC_COMPUTED:
-	case LOC_COMPUTED_ARG:
 	  fprintf_filtered (outfile, "computed at runtime");
 	  break;
 
@@ -771,154 +649,6 @@ print_symbol (void *args)
   return 1;
 }
 
-void
-maintenance_print_psymbols (char *args, int from_tty)
-{
-  char **argv;
-  struct ui_file *outfile;
-  struct cleanup *cleanups;
-  char *symname = NULL;
-  char *filename = DEV_TTY;
-  struct objfile *objfile;
-  struct partial_symtab *ps;
-
-  dont_repeat ();
-
-  if (args == NULL)
-    {
-      error ("print-psymbols takes an output file name and optional symbol file name");
-    }
-  else if ((argv = buildargv (args)) == NULL)
-    {
-      nomem (0);
-    }
-  cleanups = make_cleanup_freeargv (argv);
-
-  if (argv[0] != NULL)
-    {
-      filename = argv[0];
-      /* If a second arg is supplied, it is a source file name to match on */
-      if (argv[1] != NULL)
-	{
-	  symname = argv[1];
-	}
-    }
-
-  filename = tilde_expand (filename);
-  make_cleanup (xfree, filename);
-
-  outfile = gdb_fopen (filename, FOPEN_WT);
-  if (outfile == 0)
-    perror_with_name (filename);
-  make_cleanup_ui_file_delete (outfile);
-
-  immediate_quit++;
-  ALL_PSYMTABS (objfile, ps)
-    if (symname == NULL || strcmp (symname, ps->filename) == 0)
-    dump_psymtab (objfile, ps, outfile);
-  immediate_quit--;
-  do_cleanups (cleanups);
-}
-
-static void
-print_partial_symbols (struct partial_symbol **p, int count, char *what,
-		       struct ui_file *outfile)
-{
-  fprintf_filtered (outfile, "  %s partial symbols:\n", what);
-  while (count-- > 0)
-    {
-      fprintf_filtered (outfile, "    `%s'", DEPRECATED_SYMBOL_NAME (*p));
-      if (SYMBOL_DEMANGLED_NAME (*p) != NULL)
-	{
-	  fprintf_filtered (outfile, "  `%s'", SYMBOL_DEMANGLED_NAME (*p));
-	}
-      fputs_filtered (", ", outfile);
-      switch (SYMBOL_DOMAIN (*p))
-	{
-	case UNDEF_DOMAIN:
-	  fputs_filtered ("undefined domain, ", outfile);
-	  break;
-	case VAR_DOMAIN:
-	  /* This is the usual thing -- don't print it */
-	  break;
-	case STRUCT_DOMAIN:
-	  fputs_filtered ("struct domain, ", outfile);
-	  break;
-	case LABEL_DOMAIN:
-	  fputs_filtered ("label domain, ", outfile);
-	  break;
-	default:
-	  fputs_filtered ("<invalid domain>, ", outfile);
-	  break;
-	}
-      switch (SYMBOL_CLASS (*p))
-	{
-	case LOC_UNDEF:
-	  fputs_filtered ("undefined", outfile);
-	  break;
-	case LOC_CONST:
-	  fputs_filtered ("constant int", outfile);
-	  break;
-	case LOC_STATIC:
-	  fputs_filtered ("static", outfile);
-	  break;
-	case LOC_INDIRECT:
-	  fputs_filtered ("extern global", outfile);
-	  break;
-	case LOC_REGISTER:
-	  fputs_filtered ("register", outfile);
-	  break;
-	case LOC_ARG:
-	  fputs_filtered ("pass by value", outfile);
-	  break;
-	case LOC_REF_ARG:
-	  fputs_filtered ("pass by reference", outfile);
-	  break;
-	case LOC_REGPARM:
-	  fputs_filtered ("register parameter", outfile);
-	  break;
-	case LOC_REGPARM_ADDR:
-	  fputs_filtered ("register address parameter", outfile);
-	  break;
-	case LOC_LOCAL:
-	  fputs_filtered ("stack parameter", outfile);
-	  break;
-	case LOC_TYPEDEF:
-	  fputs_filtered ("type", outfile);
-	  break;
-	case LOC_LABEL:
-	  fputs_filtered ("label", outfile);
-	  break;
-	case LOC_BLOCK:
-	  fputs_filtered ("function", outfile);
-	  break;
-	case LOC_CONST_BYTES:
-	  fputs_filtered ("constant bytes", outfile);
-	  break;
-	case LOC_LOCAL_ARG:
-	  fputs_filtered ("shuffled arg", outfile);
-	  break;
-	case LOC_UNRESOLVED:
-	  fputs_filtered ("unresolved", outfile);
-	  break;
-	case LOC_OPTIMIZED_OUT:
-	  fputs_filtered ("optimized out", outfile);
-	  break;
-	case LOC_COMPUTED:
-	case LOC_COMPUTED_ARG:
-	  fputs_filtered ("computed at runtime", outfile);
-	  break;
-	default:
-	  fputs_filtered ("<invalid location>", outfile);
-	  break;
-	}
-      fputs_filtered (", ", outfile);
-      print_address_numeric (SYMBOL_VALUE_ADDRESS (*p), 1, outfile);
-      fprintf_filtered (outfile, "\n");
-      p++;
-    }
-}
-
 void
 maintenance_print_msymbols (char *args, int from_tty)
 {
@@ -927,18 +657,18 @@ maintenance_print_msymbols (char *args, int from_tty)
   struct cleanup *cleanups;
   char *filename = DEV_TTY;
   char *symname = NULL;
+  struct program_space *pspace;
   struct objfile *objfile;
 
+  struct stat sym_st, obj_st;
+
   dont_repeat ();
 
   if (args == NULL)
     {
-      error ("print-msymbols takes an output file name and optional symbol file name");
-    }
-  else if ((argv = buildargv (args)) == NULL)
-    {
-      nomem (0);
+      error (_("print-msymbols takes an output file name and optional symbol file name"));
     }
+  argv = gdb_buildargv (args);
   cleanups = make_cleanup_freeargv (argv);
 
   if (argv[0] != NULL)
@@ -947,7 +677,10 @@ maintenance_print_msymbols (char *args, int from_tty)
       /* If a second arg is supplied, it is a source file name to match on */
       if (argv[1] != NULL)
 	{
-	  symname = argv[1];
+	  symname = xfullpath (argv[1]);
+	  make_cleanup (xfree, symname);
+	  if (symname && stat (symname, &sym_st))
+	    perror_with_name (symname);
 	}
     }
 
@@ -960,9 +693,11 @@ maintenance_print_msymbols (char *args, int from_tty)
   make_cleanup_ui_file_delete (outfile);
 
   immediate_quit++;
-  ALL_OBJFILES (objfile)
-    if (symname == NULL || strcmp (symname, objfile->name) == 0)
-    dump_msymbols (objfile, outfile);
+  ALL_PSPACES (pspace)
+    ALL_PSPACE_OBJFILES (pspace, objfile)
+      if (symname == NULL
+	  || (!stat (objfile->name, &obj_st) && sym_st.st_ino == obj_st.st_ino))
+	dump_msymbols (objfile, outfile);
   immediate_quit--;
   fprintf_filtered (outfile, "\n\n");
   do_cleanups (cleanups);
@@ -971,13 +706,15 @@ maintenance_print_msymbols (char *args, int from_tty)
 void
 maintenance_print_objfiles (char *ignore, int from_tty)
 {
+  struct program_space *pspace;
   struct objfile *objfile;
 
   dont_repeat ();
 
   immediate_quit++;
-  ALL_OBJFILES (objfile)
-    dump_objfile (objfile);
+  ALL_PSPACES (pspace)
+    ALL_PSPACE_OBJFILES (pspace, objfile)
+      dump_objfile (objfile);
   immediate_quit--;
 }
 
@@ -986,12 +723,14 @@ maintenance_print_objfiles (char *ignore, int from_tty)
 void
 maintenance_info_symtabs (char *regexp, int from_tty)
 {
+  struct program_space *pspace;
   struct objfile *objfile;
 
   if (regexp)
     re_comp (regexp);
 
-  ALL_OBJFILES (objfile)
+  ALL_PSPACES (pspace)
+    ALL_PSPACE_OBJFILES (pspace, objfile)
     {
       struct symtab *symtab;
       
@@ -1000,209 +739,43 @@ maintenance_info_symtabs (char *regexp, int from_tty)
       int printed_objfile_start = 0;
 
       ALL_OBJFILE_SYMTABS (objfile, symtab)
-        if (! regexp
-            || re_exec (symtab->filename))
-          {
-            if (! printed_objfile_start)
-              {
-                printf_filtered ("{ objfile %s ", objfile->name);
-                wrap_here ("  ");
-                printf_filtered ("((struct objfile *) %p)\n", objfile);
-                printed_objfile_start = 1;
-              }
+	{
+	  QUIT;
 
-            printf_filtered ("  { symtab %s ", symtab->filename);
-            wrap_here ("    ");
-            printf_filtered ("((struct symtab *) %p)\n", symtab);
-            printf_filtered ("    dirname %s\n",
-                             symtab->dirname ? symtab->dirname : "(null)");
-            printf_filtered ("    fullname %s\n",
-                             symtab->fullname ? symtab->fullname : "(null)");
-            printf_filtered ("    blockvector ((struct blockvector *) %p)%s\n",
-                             symtab->blockvector,
-                             symtab->primary ? " (primary)" : "");
-            printf_filtered ("    debugformat %s\n", symtab->debugformat);
-            printf_filtered ("  }\n");
-          }
+	  if (! regexp
+	      || re_exec (symtab->filename))
+	    {
+	      if (! printed_objfile_start)
+		{
+		  printf_filtered ("{ objfile %s ", objfile->name);
+		  wrap_here ("  ");
+		  printf_filtered ("((struct objfile *) %s)\n", 
+				   host_address_to_string (objfile));
+		  printed_objfile_start = 1;
+		}
+
+	      printf_filtered ("	{ symtab %s ", symtab->filename);
+	      wrap_here ("    ");
+	      printf_filtered ("((struct symtab *) %s)\n", 
+			       host_address_to_string (symtab));
+	      printf_filtered ("	  dirname %s\n",
+			       symtab->dirname ? symtab->dirname : "(null)");
+	      printf_filtered ("	  fullname %s\n",
+			       symtab->fullname ? symtab->fullname : "(null)");
+	      printf_filtered ("	  blockvector ((struct blockvector *) %s)%s\n",
+			       host_address_to_string (symtab->blockvector),
+			       symtab->primary ? " (primary)" : "");
+	      printf_filtered ("	  linetable ((struct linetable *) %s)\n",
+			       host_address_to_string (symtab->linetable));
+	      printf_filtered ("	  debugformat %s\n", symtab->debugformat);
+	      printf_filtered ("	}\n");
+	    }
+	}
 
       if (printed_objfile_start)
         printf_filtered ("}\n");
     }
 }
-
-
-/* List all the partial symbol tables whose names match REGEXP (optional).  */
-void
-maintenance_info_psymtabs (char *regexp, int from_tty)
-{
-  struct objfile *objfile;
-
-  if (regexp)
-    re_comp (regexp);
-
-  ALL_OBJFILES (objfile)
-    {
-      struct partial_symtab *psymtab;
-
-      /* We don't want to print anything for this objfile until we
-         actually find a symtab whose name matches.  */
-      int printed_objfile_start = 0;
-
-      ALL_OBJFILE_PSYMTABS (objfile, psymtab)
-        if (! regexp
-            || re_exec (psymtab->filename))
-          {
-            if (! printed_objfile_start)
-              {
-                printf_filtered ("{ objfile %s ", objfile->name);
-                wrap_here ("  ");
-                printf_filtered ("((struct objfile *) %p)\n", objfile);
-                printed_objfile_start = 1;
-              }
-
-            printf_filtered ("  { psymtab %s ", psymtab->filename);
-            wrap_here ("    ");
-            printf_filtered ("((struct partial_symtab *) %p)\n", psymtab);
-            printf_filtered ("    readin %s\n",
-                             psymtab->readin ? "yes" : "no");
-            printf_filtered ("    fullname %s\n",
-                             psymtab->fullname ? psymtab->fullname : "(null)");
-            printf_filtered ("    text addresses ");
-            print_address_numeric (psymtab->textlow, 1, gdb_stdout);
-            printf_filtered (" -- ");
-            print_address_numeric (psymtab->texthigh, 1, gdb_stdout);
-            printf_filtered ("\n");
-            printf_filtered ("    globals ");
-            if (psymtab->n_global_syms)
-              {
-                printf_filtered ("(* (struct partial_symbol **) %p @ %d)\n",
-                                 (psymtab->objfile->global_psymbols.list
-                                  + psymtab->globals_offset),
-                                 psymtab->n_global_syms);
-              }
-            else
-              printf_filtered ("(none)\n");
-            printf_filtered ("    statics ");
-            if (psymtab->n_static_syms)
-              {
-                printf_filtered ("(* (struct partial_symbol **) %p @ %d)\n",
-                                 (psymtab->objfile->static_psymbols.list
-                                  + psymtab->statics_offset),
-                                 psymtab->n_static_syms);
-              }
-            else
-              printf_filtered ("(none)\n");
-            printf_filtered ("    dependencies ");
-            if (psymtab->number_of_dependencies)
-              {
-                int i;
-
-                printf_filtered ("{\n");
-                for (i = 0; i < psymtab->number_of_dependencies; i++)
-                  {
-                    struct partial_symtab *dep = psymtab->dependencies[i];
-
-                    /* Note the string concatenation there --- no comma.  */
-                    printf_filtered ("      psymtab %s "
-                                     "((struct partial_symtab *) %p)\n",
-                                     dep->filename, dep);
-                  }
-                printf_filtered ("    }\n");
-              }
-            else
-              printf_filtered ("(none)\n");
-            printf_filtered ("  }\n");
-          }
-
-      if (printed_objfile_start)
-        printf_filtered ("}\n");
-    }
-}
-
-
-/* Check consistency of psymtabs and symtabs.  */
-
-void
-maintenance_check_symtabs (char *ignore, int from_tty)
-{
-  struct symbol *sym;
-  struct partial_symbol **psym;
-  struct symtab *s = NULL;
-  struct partial_symtab *ps;
-  struct blockvector *bv;
-  struct objfile *objfile;
-  struct block *b;
-  int length;
-
-  ALL_PSYMTABS (objfile, ps)
-  {
-    s = PSYMTAB_TO_SYMTAB (ps);
-    if (s == NULL)
-      continue;
-    bv = BLOCKVECTOR (s);
-    b = BLOCKVECTOR_BLOCK (bv, STATIC_BLOCK);
-    psym = ps->objfile->static_psymbols.list + ps->statics_offset;
-    length = ps->n_static_syms;
-    while (length--)
-      {
-	sym = lookup_block_symbol (b, DEPRECATED_SYMBOL_NAME (*psym),
-				   NULL, SYMBOL_DOMAIN (*psym));
-	if (!sym)
-	  {
-	    printf_filtered ("Static symbol `");
-	    puts_filtered (DEPRECATED_SYMBOL_NAME (*psym));
-	    printf_filtered ("' only found in ");
-	    puts_filtered (ps->filename);
-	    printf_filtered (" psymtab\n");
-	  }
-	psym++;
-      }
-    b = BLOCKVECTOR_BLOCK (bv, GLOBAL_BLOCK);
-    psym = ps->objfile->global_psymbols.list + ps->globals_offset;
-    length = ps->n_global_syms;
-    while (length--)
-      {
-	sym = lookup_block_symbol (b, DEPRECATED_SYMBOL_NAME (*psym),
-				   NULL, SYMBOL_DOMAIN (*psym));
-	if (!sym)
-	  {
-	    printf_filtered ("Global symbol `");
-	    puts_filtered (DEPRECATED_SYMBOL_NAME (*psym));
-	    printf_filtered ("' only found in ");
-	    puts_filtered (ps->filename);
-	    printf_filtered (" psymtab\n");
-	  }
-	psym++;
-      }
-    if (ps->texthigh < ps->textlow)
-      {
-	printf_filtered ("Psymtab ");
-	puts_filtered (ps->filename);
-	printf_filtered (" covers bad range ");
-	print_address_numeric (ps->textlow, 1, gdb_stdout);
-	printf_filtered (" - ");
-	print_address_numeric (ps->texthigh, 1, gdb_stdout);
-	printf_filtered ("\n");
-	continue;
-      }
-    if (ps->texthigh == 0)
-      continue;
-    if (ps->textlow < BLOCK_START (b) || ps->texthigh > BLOCK_END (b))
-      {
-	printf_filtered ("Psymtab ");
-	puts_filtered (ps->filename);
-	printf_filtered (" covers ");
-	print_address_numeric (ps->textlow, 1, gdb_stdout);
-	printf_filtered (" - ");
-	print_address_numeric (ps->texthigh, 1, gdb_stdout);
-	printf_filtered (" but symtab covers only ");
-	print_address_numeric (BLOCK_START (b), 1, gdb_stdout);
-	printf_filtered (" - ");
-	print_address_numeric (BLOCK_END (b), 1, gdb_stdout);
-	printf_filtered ("\n");
-      }
-  }
-}
 
 
 /* Return the nexting depth of a block within other blocks in its symtab.  */
@@ -1211,6 +784,7 @@ static int
 block_depth (struct block *block)
 {
   int i = 0;
+
   while ((block = BLOCK_SUPERBLOCK (block)) != NULL)
     {
       i++;
@@ -1219,35 +793,6 @@ block_depth (struct block *block)
 }
 
 
-/* Increase the space allocated for LISTP, which is probably
-   global_psymbols or static_psymbols. This space will eventually
-   be freed in free_objfile().  */
-
-void
-extend_psymbol_list (struct psymbol_allocation_list *listp,
-		     struct objfile *objfile)
-{
-  int new_size;
-  if (listp->size == 0)
-    {
-      new_size = 255;
-      listp->list = (struct partial_symbol **)
-	xmalloc (new_size * sizeof (struct partial_symbol *));
-    }
-  else
-    {
-      new_size = listp->size * 2;
-      listp->list = (struct partial_symbol **)
-	xrealloc ((char *) listp->list,
-		  new_size * sizeof (struct partial_symbol *));
-    }
-  /* Next assumes we only went one over.  Should be good if
-     program works correctly */
-  listp->next = listp->list + listp->size;
-  listp->size = new_size;
-}
-
-
 /* Do early runtime initializations. */
 void
 _initialize_symmisc (void)
