@@ -1,4 +1,4 @@
-/* Copyright 1992, 1993, 1994, 1995, 1996, 1999, 2007
+/* Copyright 1992, 1993, 1994, 1995, 1996, 1999, 2007, 2008, 2009, 2010
    Free Software Foundation, Inc.
 
    This file is part of GDB.
@@ -96,6 +96,8 @@ mmapdata ()
 	  return;
 	}
     }
+  /* Touch buf2 so kernel writes it out into 'core'. */
+  buf2[0] = buf1[0];
 }
 
 void
@@ -137,4 +139,3 @@ int main ()
   func1 ();
   return 0;
 }
-
