@@ -1,12 +1,13 @@
 /* Traditional frame unwind support, for GDB the GNU Debugger.
 
-   Copyright (C) 2003, 2004 Free Software Foundation, Inc.
+   Copyright (C) 2003, 2004, 2007, 2008, 2009, 2010
+   Free Software Foundation, Inc.
 
    This file is part of GDB.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -15,31 +16,31 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #include "defs.h"
 #include "frame.h"
 #include "trad-frame.h"
 #include "regcache.h"
+#include "frame-unwind.h"
+#include "value.h"
 
 struct trad_frame_cache
 {
-  struct frame_info *next_frame;
+  struct frame_info *this_frame;
   CORE_ADDR this_base;
   struct trad_frame_saved_reg *prev_regs;
   struct frame_id this_id;
 };
 
 struct trad_frame_cache *
-trad_frame_cache_zalloc (struct frame_info *next_frame)
+trad_frame_cache_zalloc (struct frame_info *this_frame)
 {
   struct trad_frame_cache *this_trad_cache;
 
   this_trad_cache = FRAME_OBSTACK_ZALLOC (struct trad_frame_cache);
-  this_trad_cache->prev_regs = trad_frame_alloc_saved_regs (next_frame);
-  this_trad_cache->next_frame = next_frame;
+  this_trad_cache->prev_regs = trad_frame_alloc_saved_regs (this_frame);
+  this_trad_cache->this_frame = this_frame;
   return this_trad_cache;
 }
 
@@ -49,13 +50,14 @@ trad_frame_cache_zalloc (struct frame_info *next_frame)
    for all potential instruction sequences).  */
 
 struct trad_frame_saved_reg *
-trad_frame_alloc_saved_regs (struct frame_info *next_frame)
+trad_frame_alloc_saved_regs (struct frame_info *this_frame)
 {
   int regnum;
-  struct gdbarch *gdbarch = get_frame_arch (next_frame);
-  int numregs = NUM_REGS + NUM_PSEUDO_REGS;
+  struct gdbarch *gdbarch = get_frame_arch (this_frame);
+  int numregs = gdbarch_num_regs (gdbarch) + gdbarch_num_pseudo_regs (gdbarch);
   struct trad_frame_saved_reg *this_saved_regs
     = FRAME_OBSTACK_CALLOC (numregs, struct trad_frame_saved_reg);
+
   for (regnum = 0; regnum < numregs; regnum++)
     {
       this_saved_regs[regnum].realreg = regnum;
@@ -130,66 +132,33 @@ trad_frame_set_unknown (struct trad_frame_saved_reg this_saved_regs[],
   this_saved_regs[regnum].addr = -1;
 }
 
-void
-trad_frame_get_prev_register (struct frame_info *next_frame,
+struct value *
+trad_frame_get_prev_register (struct frame_info *this_frame,
 			      struct trad_frame_saved_reg this_saved_regs[],
-			      int regnum, int *optimizedp,
-			      enum lval_type *lvalp, CORE_ADDR *addrp,
-			      int *realregp, gdb_byte *bufferp)
+			      int regnum)
 {
-  struct gdbarch *gdbarch = get_frame_arch (next_frame);
   if (trad_frame_addr_p (this_saved_regs, regnum))
-    {
-      /* The register was saved in memory.  */
-      *optimizedp = 0;
-      *lvalp = lval_memory;
-      *addrp = this_saved_regs[regnum].addr;
-      *realregp = -1;
-      if (bufferp != NULL)
-	{
-	  /* Read the value in from memory.  */
-	  get_frame_memory (next_frame, this_saved_regs[regnum].addr, bufferp,
-			    register_size (gdbarch, regnum));
-	}
-    }
+    /* The register was saved in memory.  */
+    return frame_unwind_got_memory (this_frame, regnum,
+				    this_saved_regs[regnum].addr);
   else if (trad_frame_realreg_p (this_saved_regs, regnum))
-    {
-      *optimizedp = 0;
-      *lvalp = lval_register;
-      *addrp = 0;
-      *realregp = this_saved_regs[regnum].realreg;
-      /* Ask the next frame to return the value of the register.  */
-      if (bufferp)
-	frame_unwind_register (next_frame, (*realregp), bufferp);
-    }
+    return frame_unwind_got_register (this_frame, regnum,
+				      this_saved_regs[regnum].realreg);
   else if (trad_frame_value_p (this_saved_regs, regnum))
-    {
-      /* The register's value is available.  */
-      *optimizedp = 0;
-      *lvalp = not_lval;
-      *addrp = 0;
-      *realregp = -1;
-      if (bufferp != NULL)
-	store_unsigned_integer (bufferp, register_size (gdbarch, regnum),
-				this_saved_regs[regnum].addr);
-    }
+    /* The register's value is available.  */
+    return frame_unwind_got_constant (this_frame, regnum,
+				      this_saved_regs[regnum].addr);
   else
-    {
-      error (_("Register %s not available"),
-	     gdbarch_register_name (gdbarch, regnum));
-    }
+    return frame_unwind_got_optimized (this_frame, regnum);
 }
 
-void
+struct value *
 trad_frame_get_register (struct trad_frame_cache *this_trad_cache,
-			 struct frame_info *next_frame,
-			 int regnum, int *optimizedp,
-			 enum lval_type *lvalp, CORE_ADDR *addrp,
-			 int *realregp, gdb_byte *bufferp)
+			 struct frame_info *this_frame,
+			 int regnum)
 {
-  trad_frame_get_prev_register (next_frame, this_trad_cache->prev_regs,
-				regnum, optimizedp, lvalp, addrp, realregp,
-				bufferp);
+  return trad_frame_get_prev_register (this_frame, this_trad_cache->prev_regs,
+				       regnum);
 }
 
 void
