@@ -256,10 +256,15 @@ tui_show_locator_content (void)
 
       string = tui_make_status_line (&element->which_element.locator);
       wmove (locator->handle, 0, 0);
-      wstandout (locator->handle);
+      /* We ignore the return value from wstandout and wstandend, casting
+	 them to void in order to avoid a compiler warning.  The warning
+	 itself was introduced by a patch to ncurses 5.7 dated 2009-08-29,
+	 changing these macro to expand to code that causes the compiler
+	 to generate an unused-value warning.  */
+      (void) wstandout (locator->handle);
       waddstr (locator->handle, string);
       wclrtoeol (locator->handle);
-      wstandend (locator->handle);
+      (void) wstandend (locator->handle);
       tui_refresh_win (locator);
       wmove (locator->handle, 0, 0);
       xfree (string);
@@ -350,6 +355,7 @@ tui_show_frame_info (struct frame_info *fi)
       for (i = 0; i < (tui_source_windows ())->count; i++)
 	{
 	  union tui_which_element *item;
+
 	  win_info = (tui_source_windows ())->list[i];
 
 	  item = &((struct tui_win_element *) locator->content[0])->which_element;
@@ -373,6 +379,7 @@ tui_show_frame_info (struct frame_info *fi)
 	  if (win_info == TUI_SRC_WIN)
 	    {
 	      struct tui_line_or_address l;
+
 	      l.loa = LOA_LINE;
 	      l.u.line_no = start_line;
 	      if (!(source_already_displayed
@@ -390,6 +397,7 @@ tui_show_frame_info (struct frame_info *fi)
 	      if (win_info == TUI_DISASM_WIN)
 		{
 		  struct tui_line_or_address a;
+
 		  a.loa = LOA_ADDRESS;
 		  a.u.addr = low;
 		  if (!tui_addr_is_displayed (item->locator.addr, win_info, TRUE))
